class MessagesController < ApplicationController
  before_filter :login_required

	# GET /messages
  # GET /messages.xml
  def index
		if params['box'].eql?'outbox'
			@messages = Message.find_all_by_sender_id(current_user,
				:conditions => 'deleted_on_sender=false')
			@box = 'outbox'
		else
			params['box'] = 'inbox'
			@messages = Message.find_all_by_receiver_id(current_user, 
				:conditions => 'deleted_on_receiver=false')
			@box = 'inbox'
		end
  end

	# GET /messages/1
	# GET /messages/1.xml
	def show
		@message = Message.find(params[:id])
		@box = 'inbox' if @message.receiver==current_user
		@box = 'outbox' if @message.sender==current_user
		@message.read = true if @box.eql?'inbox'
		@message.save
	end

	# GET /messages/new
	# GET /messages/new.xml
	def new
		@message = Message.new
		# reply
		if params[:id]
			@original = Message.find(params[:id])
			@message.receiver = @original.sender
			@message.subject = "Re: #{@original.subject}"
			@message.body = @original.body.gsub("\n", "\n> ")
			@message.body = "#{@original.sender.username} on #{@original.created_at.to_formatted_s(:long)}:\n> #{@message.body}"
		end
		#direct message
		if params[:user_id]
		  @user = User.find(params[:user_id])
	  end
	end

	# POST /messages
	# POST /messages.xml
	def create
		params[:message][:sender_id] = current_user.id
		@message = Message.new(params[:message])
		if @message.save
			flash[:notice] = "Message was successfully sent."
			redirect_to outbox_url
		else
			render :action => 'new'
		end
	end
	
  # PUT /messages/1
  # PUT /messages/1.xml
  def update
    @message = Message.find(params[:id])

    respond_to do |format|
      if @message.update_attributes(params[:message])
        flash[:notice] = 'Message was successfully updated.'
        format.html { redirect_to(@message) }
        format.xml  { head :ok }
      else
        format.html { render :action => "edit" }
        format.xml  { render :xml => @message.errors, :status => :unprocessable_entity }
      end
    end
  end

  # DELETE /messages/1
  # DELETE /messages/1.xml
	def destroy
		@message = Message.find(params[:id])
		if current_user.id==@message.sender.id
			@message.deleted_on_sender = true
			@box = 'outbox'
		elsif current_user.id==@message.receiver.id
			@message.deleted_on_receiver = true
			@box = 'inbox'
		end
		
		if @message.deleted_on_sender && @message.deleted_on_receiver
			@message.destroy
		else
			@message.save
		end
		if !params[:multiple]
			if @box.eql?'inbox'
				redirect_to inbox_url
			else
				redirect_to outbox_url
			end
		end
		return @box
	end

	def delete_checked
		params.each do |par|
			if par[0].to_i>0
				params[:id] = par[0]
				params[:multiple] = true
				@box = self.destroy
			end
		end
		flash[:notice] = "Checked messages deleted."
		if @box.eql?'inbox'
			redirect_to inbox_url
		else
			redirect_to outbox_url
		end
	end
end
