class SessionsController < ApplicationController
  # If you want "remember me" functionality, add this before_filter to Application Controller
  # before_filter :login_from_cookie
	
	# login
  def create
    if using_open_id?
      open_id_authentication(params[:openid_url])
    else
      password_authentication(params[:email], params[:password])
    end
  end
	
	# logout
	def destroy
		self.current_user.forget_me if logged_in?
		cookies.delete :auth_token
		reset_session
		redirect_to('/')
	end

	def forgot_password
		return unless request.post?
		if @user = User.find_by_email(params[:email])
			@user.forgot_password
			@user.save
			redirect_to :controller => :sessions, :action => 'forgot_password'
			flash[:notice] = _("An email with further instructions was sent for your address.")
		else
			flash[:error] = _("Email address not found.")
		end
	end

	def reset_password
		@user = User.find_by_password_reset_code(params[:id])
		raise if @user.nil?
		return if @user unless params[:password]
		if (params[:password] == params[:password_confirmation])
			self.current_user = @user #for the next two lines to work
			current_user.password_confirmation = params[:password_confirmation]
			current_user.password = params[:password]
			@user.reset_password
		else
			flash[:error] = _("Password mismatch")
		end
		#TODO este redirect leva-me a sitio incerto! Era mais lindo uma mensagem
		redirect_to session_url
		rescue
			flash[:error] = _("The reset code is incorrect. Check if the URL is the one sent to your email.")
			redirect_to :controller => :sessions, :action => 'reset_error'
	end

	protected
  def open_id_authentication(openid_url)
    authenticate_with_open_id(openid_url, :required => [:nickname, :email, :dob, :fullname, :gender]) do |result, identity_url, registration|
      if result.successful?
        @user = User.find_or_initialize_by_identity_url(identity_url)
        if @user.new_record?
          @user.gender = registration['gender'].equal?"M"
          @user.identity_url = identity_url
          @user.name = registration['fullname']
          @user.surname = registration['fullname']
          @user.birth_date = registration['dob']
          @user.username = registration['nickname']
          @user.email = registration['email']
          @user.password = 'dummy'
          @user.password_confirmation = 'dummy'
          @user.role = Role.find_by_name('User')
          @user.save!
        end
        self.current_user = @user
        successful_login
      else
        failed_login result.message
      end
    end

    rescue ActiveRecord::RecordInvalid
    flash.now[:error] = _("Could not use that OpenID! Read why below.")
    render :action => 'login'
  end

  def password_authentication(email, password)
    self.current_user = User.authenticate(email, password)
    if logged_in?
      successful_login
    else
      failed_login _("Invalid username or password.")
    end
  end

	def failed_login(message)
		flash[:error] = message
		render :action => 'login'
	end
	
	def successful_login
		if params[:remember_me] == "1"
			self.current_user.remember_me
			cookies[:auth_token] = {:value => self.current_user.remember_token , :expires => self.current_user.remember_token_expires_at}
		end
		redirect_to('/')
		flash[:notice] = _("Logged in successfully")
  end
end
