class Ad < ActiveRecord::Base
	acts_as_mappable
	
	validates_presence_of :description, :title, :price, :available_at
  validates_length_of :title, :within => (5..35)
  
  belongs_to :user
  belongs_to :region
  has_many :comments, :dependent => :destroy
	has_many :images, :dependent => :destroy
	has_many :reviews, :dependent => :destroy
	
	before_save :save_hash
	
	def first_image
		image = Image.find_by_ad_id(self.id, :order => 'num ASC')
		if image.nil?
			image = Image.no_image(self)
		end
		return image
	end

	def destroy_imagedir
		dir = "#{RAILS_ROOT}/public/uploads/images/#{self.id}"
		FileUtils.remove_dir(dir) if File.exists?dir
	end
	
	def save_hash
	  if !self.sms_hash
	    hash = Array.new(2) { rand(256) }.pack('C*').unpack('H*').first
	    self.sms_hash = hash
    end
  end
	
	def after_destroy
		destroy_imagedir
	end
	
	def facilities
	  f = []
	  f.push(Ad.human_attribute_name("fourniture")) if self.fourniture
	  f.push(Ad.human_attribute_name("television")) if self.television
    f.push(Ad.human_attribute_name("private_bathroom")) if self.private_bathroom
    f.push(Ad.human_attribute_name("washing_machine")) if self.washing_machine
    f.push(Ad.human_attribute_name("internet")) if self.internet
    f.push(Ad.human_attribute_name("cable_tv")) if self.cable_tv
    f.push(Ad.human_attribute_name("oven")) if self.oven
    f.push(Ad.human_attribute_name("microwave")) if self.microwave_oven
    f.push(Ad.human_attribute_name("fridge")) if self.fridge
    return f
  end
	
	def activate
	  self.active = true
	  self.validated_at = Time.now
  end
	
	def search_optimized(params = {})
    cond = []
    cond << ["name LIKE ?", params[:name] + '%'] unless params[:name].blank?
    cond << ["email = ?",   params[:email]]       unless params[:email].blank?
    cond << ["gender = ?",  params[:gender]]      unless params[:gender].blank?
    conditions = cond.map { |c| merge_conditions(c) }.join(' AND ')
    find(:all, :conditions => conditions)
  end
	
	def self.advanced_search(params)
	  region = params[:region] ? "=#{params[:region]}" : ">0"
	  category = params[:category] ? "=#{params[:category]}" : ">0"
    conditions = "region_id#{region}"
    if (!params[:category] || params[:category].empty?)
      ads = Ad.find(:all, :conditions => conditions, :order => "id DESC")
    else
      ads = []
      region = Region.find(params[:region])
      poiconditions = "category_id#{category}"
      if (params[:poi] && params[:poi].to_i>0)
        pois = Poi.find_all_by_id(params[:poi]) # :all used for pois.each
      else
        pois = Poi.find(:all, :origin =>[region.lat,region.lng], :conditions => poiconditions)
      end
      pois.each do |poi|
        ads << Ad.find(:all, :origin => [poi.lat,poi.lng], :conditions => conditions, :limit => 30)
      end
      ads.flatten!
      return ads.sort_by{|a| a[:distance]}
    end
    return ads
	end
	
end
