require "RMagick"
class Image < ActiveRecord::Base
	$IMAGE_TYPES = ['image/jpeg', 'image/pjpeg', 'image/gif', 'image/png', 'image/x-png']
	$SIZES = ['110x88', '220x176']
	
	before_save :validate_image_type
	before_save :validate_image_size
	before_save :create_images
	
	belongs_to :ad
	attr_accessor :up_file
	
	def self.sizes
		return $SIZES
	end
	
	def self.image_types
		return $IMAGE_TYPES
	end
	
	def image_dir
		self.ad.nil? ? "/uploads/images" : "/uploads/images/#{self.ad.id}"
  end

	def image_absdir
		return "#{RAILS_ROOT}/public#{self.image_dir}"
	end
	
	def src(size=Image.sizes.first, just_file=false)
		Image.sizes.each do |s|
			if s.eql?(size)
				if just_file
					return "#{s}_#{self.name}"
				else
					return "#{self.image_dir}/#{s}_#{self.name}"
				end
			end
		end
	end
	
	def absrc
	  return "http://localhost:3000/"+self.src(Image.sizes.last)
  end
	
	# cria um 'no_image' para o ad_id
	def self.no_image(ad)
		img = self.new
		img.name = "no_photo.gif"
		return img
	end
	
	def create_images
		unless @up_file.nil? || @up_file.to_s.empty?
			self.name = unique_name("image.jpg")
			FileUtils.makedirs(self.image_absdir)
			File.open("#{self.image_absdir}/_#{self.name}", "wb") { |f| f.write(@up_file.read) }
			
			# criar imagens de tamanhos definidos
			Image.sizes.each do |s|
				@temp = Magick::Image.read("#{self.image_absdir}/_#{self.name}").first
				out = @temp.change_geometry("#{s}"){ |cols, rows, img| img.resize!(cols, rows)}
				out.write(self.image_absdir+"/#{s}_#{self.name}")
			end
			
			# apagar a original
			FileUtils.rm_r("#{self.image_absdir}/_#{self.name}")
		end
  end
	
	def before_destroy
		Image.sizes.each do |s|
			FileUtils.rm_r("#{self.image_absdir}/#{s}_#{self.name}")
		end
	end
	
	# validates the content type of the uploaded image
	def validate_image_type
		unless @up_file.nil? || @up_file.to_s.empty?
			if Image.image_types.include?(@up_file.content_type.strip)
				true
			else
				errors.add("image", "A imagem não tem um formato válido.")
				false
			end
		end
	end
	
	# validates the size of the image
	def validate_image_size
		unless @up_file.nil?
			if @up_file.size > 2097152
				errors.add("image", "A imagem não pode ter tamanho superior a 2MB")
				false
			else
				true
			end
		end
	end
	
	private
	def unique_name(filename)
		filename = filename.split(".")
		ext = filename.last
		filename.delete_at(filename.length-1)
		filename = filename.first
		newname = filename
		i = 1
		while File.exist?"#{self.image_absdir}/#{Image.sizes.first}_#{newname}.#{ext}"
			newname = "#{filename}#{i.to_s}s"
			i += 1
		end
		return "#{newname}.#{ext}"
	end
	
end
