ActionController::Routing::Routes.draw do |map|  
  map.root :controller => 'statics', :action => 'entrance'
  
  # some fast routing
	map.login '/login',					  :controller => 'sessions', :action => 'login'
	map.signup '/signup',         :controller => 'users', :action => 'new'
	map.logout '/logout',			    :controller => 'sessions', :action => 'destroy'
	map.connect '/aluguer',			  :controller => 'statics', :action => 'entrance'
	map.connect '/partilha',		  :controller => 'statics', :action => 'entrance'
	map.connect '/profile/:id',	  :controller => 'users', :action => 'edit'
	map.search '/search',         :controller => 'statics', :action => 'rooms'
	map.rooms '/rooms',           :controller => 'statics', :action => 'search'
	map.connect '/rooms/:region', :controller => 'statics', :action => 'search'
	map.contact '/contact',       :controller => 'statics', :action => 'contact'
	map.privacy '/privacy',       :controller => 'statics', :action => 'privacy'
	map.terms '/terms',           :controller => 'statics', :action => 'terms'
	
	# gets sms for validation
	map.connect '/getsms',				:controller => 'ads', :action => 'getsms'
	
	# messages routes
	map.resources :messages
	map.outbox 'outbox', :controller => "messages", :action => "index", :box => "outbox"
	map.inbox 'inbox', :controller => "messages", :action => "index", :box => "inbox"
	map.reply '/messages/reply/:id', :controller => "messages", :action => "new"
	map.message_to '/messages/new/:user_id', :controller => "messages", :action => "new"
	map.delete_all 'delete_checked', :controller => "messages", :action => "delete_checked", :requirements => {:method => :post}
	
  # account
  map.namespace :account do |account|
    account.root :controller => 'ads', :action => 'index'
    account.profile 'profile', :controller => 'users', :action => 'edit'
    account.pois_import 'pois/import', :controller => 'pois', :action => 'import'
    account.pois_categories 'pois/categories', :controller => 'categories', :action => 'index'
    account.resources :categories
    account.resources :users
    account.resources :pois
    account.resources :ads, :has_many => [:reviews, :comments]
  end
	
	# ads routes
	map.location '/ads/:ad_id/location', :controller => 'ads', :action => 'location'
	map.reviews '/ads/:ad_id/reviews', :controller => 'ads', :action => 'reviews'
	
	# resources
	map.resources :users
	map.resources :comments
	map.resources :roles, :has_many => :users	
  map.resources :ads, :has_many => [:reviews, :comments]
	map.resources :regions, :has_many => :ads
	
	# open-id
	map.open_id_complete 'session', :controller => "sessions", :action => "create", :requirements => {:method => :get}
	map.resource :session
	
	map.connect ':controller/:action/:id'
  map.connect ':controller/:action/:id.:format'
end
