namespace :db do
  FIXTURES = {
    'development' => :seed_development,
    'production' => :seed_production,
  }
  
  FIXTURES.each_pair do |name, fixture|
    desc "Seed database with #{name} fixtures from db/fixtures/#{name}"
    task fixture => :environment do
      if !ARGV[1]
        puts "Usage: rake #{ARGV[0]} [all | fixture name]"
        exit(0)
      end
      
      require 'active_record/fixtures'
      # run all
      if ARGV[1].eql?'all'
        Rake::Task["db:seed_shared"].invoke
        puts "Loading all #{name} fixtures..."
        Dir.glob(RAILS_ROOT+"/db/fixtures/#{name}/*.yml").each do |file|
          puts "- "+File.basename(file, '.*').capitalize
          Fixtures.create_fixtures("db/fixtures/#{name}", File.basename(file, '.*'))
        end
        exit(0)
      else
        Rake::Task["db:seed_shared"].invoke
        puts "Loading #{name} #{ARGV[1]} fixture..."
        Dir.glob(RAILS_ROOT+"/db/fixtures/#{name}/#{ARGV[1]}.yml").each do |file|
          puts "- "+File.basename(file, '.*').capitalize
          Fixtures.create_fixtures("db/fixtures/#{name}", File.basename(file, '.*'))
        end
        exit(0)
      end
    end
  end
  
  desc "Seed database with all shared fixtures from db/fixtures/shared"
  task :seed_shared => :environment do
    require 'active_record/fixtures'
    # run all
    if ARGV[1].eql?'all'
      puts "Loading all shared fixtures..."
      Dir.glob(RAILS_ROOT+"/db/fixtures/shared/*.yml").each do |file|
        puts "- "+File.basename(file, '.*').capitalize
        Fixtures.create_fixtures("db/fixtures/shared", File.basename(file, '.*'))
      end
    else
      puts "Loading shared #{ARGV[1]} fixture..."
      Dir.glob(RAILS_ROOT+"/db/fixtures/shared/#{ARGV[1]}.yml").each do |file|
        puts "- "+File.basename(file, '.*').capitalize
        Fixtures.create_fixtures("db/fixtures/shared", File.basename(file, '.*'))
      end
    end
  end
  
end