// Global variables
var myMarker;
var map;
var geo;

/* Initializes map for marking */
function initialize_edit(type) {
	map = new google.maps.Map2(document.getElementById("map"));
	// nothing marked
	if (!Coords.lat) {
		map.setCenter(new google.maps.LatLng(38.707163, -9.135517), 8, G_HYBRID_MAP);
	}
	// already marked location
	else {
		map.setCenter(new google.maps.LatLng(Coords.lat, Coords.lng), 16, G_HYBRID_MAP);
	}
	map.enableInfoWindow();
	map.addControl(new GSmallZoomControl());
	map.addControl(new GMapTypeControl());
	
	// marked room
	if (Coords.lat) {
		var point = new GLatLng(Coords.lat, Coords.lng);
		myMarker = createMyMarker(point, map, type);
	}
	
	geo = new GClientGeocoder();
	
	GEvent.addListener(map,"click",function(overlay, point){
		if (!overlay) {
			if (myMarker) map.removeOverlay(myMarker);
			myMarker = createMyMarker(point, map, type);
		}
	}); 
}

/* Initializes map with room marker */
function initialize_show(lat, lng) {
	// only if not loaded yet
	if (!map) {
		map = new google.maps.Map2(document.getElementById("map"));
		map.setCenter(new google.maps.LatLng(lat,lng), 16, G_HYBRID_MAP);
		map.enableInfoWindow();
		map.addControl(new GSmallZoomControl());
		map.addControl(new GMapTypeControl());
		
		// add room marker
		var point = new GLatLng(lat, lng);
		var marker = createMarker(point, map);
	}
	else {
		map.setCenter(new google.maps.LatLng(lat,lng), 16, G_HYBRID_MAP);
	}
}

/* Creates fixed marker */
function createMarker(point, map, type) {
	var baseIcon = new GIcon();
	baseIcon.iconSize = new GSize(32,32);
	baseIcon.shadowSize = new GSize(56,32);
	baseIcon.iconAnchor = new GPoint(16,32);
	baseIcon.infoWindowAnchor = new GPoint(16,0);
  if (type=='poi') {
	  var roomIcon = new GIcon(baseIcon, "http://maps.google.com/mapfiles/kml/pal2/icon5.png", null, "http://maps.google.com/mapfiles/kml/pal2/icon5s.png");
  }
  else {
	  var roomIcon = new GIcon(baseIcon, "http://maps.google.com/mapfiles/kml/pal2/icon20.png", null, "http://maps.google.com/mapfiles/kml/pal2/icon20s.png");
  }
	var marker = new GMarker(point, {draggable:false,icon:roomIcon});

	// add marker to the map
	map.addOverlay(marker);
	return marker;
}

/* Creates draggable marker */
function createMyMarker(point, map, type) {

	// configure room marker
	var baseIcon = new GIcon();
	baseIcon.iconSize = new GSize(32,32);
	baseIcon.shadowSize = new GSize(56,32);
	baseIcon.iconAnchor = new GPoint(16,32);
	baseIcon.infoWindowAnchor = new GPoint(16,0);
  if (type=='poi') {
	  var roomIcon = new GIcon(baseIcon, "http://maps.google.com/mapfiles/kml/pal2/icon5.png", null, "http://maps.google.com/mapfiles/kml/pal2/icon5s.png");
  }
  else {
	  var roomIcon = new GIcon(baseIcon, "http://maps.google.com/mapfiles/kml/pal2/icon20.png", null, "http://maps.google.com/mapfiles/kml/pal2/icon20s.png");
  }
	var marker = new GMarker(point,{draggable:true,icon:roomIcon});
	
	GEvent.addListener(marker,"dragend",function(overlay, point){
		if (!overlay) {
			// fill the hidden fields again
			saveLocation(marker.getPoint());
		}
	});
	
	// add marker to map
	map.addOverlay(marker);
	lastmarker = marker;
	
	// fill the hidden fields when created
	saveLocation(point);

	return marker;
}

/* Saves the location to hidden fields */
function saveLocation(point) {
	$('ad_lat').value = point.lat();
	$('ad_lng').value = point.lng();
}

/* Tries to find the geolocation of an address */
function getAddress(search) {

	var reasons = [];
	reasons[G_GEO_SUCCESS]            = "Success";
	reasons[G_GEO_MISSING_ADDRESS]    = "Missing Address: The address was either missing or had no value.";
	reasons[G_GEO_UNKNOWN_ADDRESS]    = "Unknown Address:  No corresponding geographic location could be found for the specified address.";
	reasons[G_GEO_UNAVAILABLE_ADDRESS]= "Unavailable Address:  The geocode for the given address cannot be returned due to legal or contractual reasons.";
	reasons[G_GEO_BAD_KEY]            = "Bad Key: The API key is either invalid or does not match the domain for which it was given";
	reasons[G_GEO_TOO_MANY_QUERIES]   = "Too Many Queries: The daily geocoding quota for this site has been exceeded.";
	reasons[G_GEO_SERVER_ERROR]       = "Server error: The geocoding request could not be successfully processed.";
	reasons[403]                      = "Error 403: Probably an incorrect error caused by a bug in the handling of invalid JSON.";
	
	geo.getLocations(search, function (result) {
		if (result.Status.code == G_GEO_SUCCESS) {
			// lets assume that the first marker is the one we want
			var p = result.Placemark[0].Point.coordinates;
			var lat=p[1];
			var lng=p[0];
            
			// display the results in Map
			map.setCenter(new GLatLng(lat,lng), 16);
			
			// show message
			$('geolocation_message').innerHTML = "You can now click on map to point your room location.";
			$('geolocation_message').show();
			
		}
		else {
			if (result.Status.code == G_GEO_TOO_MANY_QUERIES) {
				nextAddress--;
				delay++;
			}
			else {
				var reason="Code "+result.Status.code;
				if (reasons[result.Status.code]) {
					reason = reasons[result.Status.code]
				}
				$('geolocation_error').innerHTML = reason;
				$('geolocation_error').show();
			}   
		}
	});
}

/* Called by form to search address */
function searchAddress() {
	$('geolocation_error').hide();
	$('geolocation_message').hide();
	var region = $("ad_region_id").options[$("ad_region_id").selectedIndex].text;
	var address = $("ad_street").value;
	var search_string = region+", Portugal, "+address;
	getAddress(search_string);
}