require File.dirname(__FILE__) + '/../test_helper'

class MessageTest < ActiveSupport::TestCase
	fixtures :users
	
	def test_message_should_have_body
    message = create_message(:body => "nil")
    assert message.errors.on(:body)
  end
	
	def test_body_should_be_shorter_than_1000
		text = ""
		1001.times do |i|
			text += "A"
		end
		message = create_message(:body => text)
		assert !message.valid?, "Message with body>1000 chars was created successfully"
  end
  
  def test_body_should_be_larger_than_3
		text = ""
		2.times do |i|
			text += "A"
		end
		message = create_message(:body => text)
		assert !message.valid?, "Message with body<3 chars was created successfully"
  end
  
  def test_sender_must_not_be_equal_to_receiver
    message = create_message(:sender => users(:aaron), :receiver => users(:aaron))
    assert !message.valid?, "Message with sender=receiver was created successfully"
  end
  
  def test_new_messages_should_be_correct
    message = create_message(:sender => users(:aaron), :receiver => users(:quentin))
    message.clone.save
    message.clone.save
    assert_equal(3, messages_for(users(:quentin)))
    message.read = true
    message.save
    assert_equal(2, messages_for(users(:quentin)))
  end

  protected
	def create_message(options = {})
		Message.create({
			:sender => users(:quentin),
			:receiver => users(:aaron),
			:read => false,
			:replied => false,
			:subject => 'this is a valid subject',
			:body => 'this is a valid body',
			:deleted_on_sender => false,
			:deleted_on_receiver => false
			}.merge(options))
	end
	
	def messages_for(user)
	  Message.find_all_by_receiver_id(user.id, :conditions => 'read=false').length
  end
end
