require File.dirname(__FILE__) + '/../test_helper'

class UserTest < Test::Unit::TestCase
  fixtures :roles, :users

  def test_should_create_user
    assert_difference 'User.count' do
      user = create_user
      assert !user.new_record?, "#{user.errors.full_messages.to_sentence}"
    end
  end

  def test_should_require_name
    assert_no_difference 'User.count' do
      u = create_user(:name => nil)
      assert u.errors.on(:name)
    end
  end

  def test_should_require_username
    assert_no_difference 'User.count' do
      u = create_user(:username => nil)
      assert u.errors.on(:username)
    end
  end

  def test_should_require_password
    assert_no_difference 'User.count' do
      u = create_user(:password => nil)
      assert u.errors.on(:password)
    end
  end

  def test_should_require_password_confirmation
    assert_no_difference 'User.count' do
      u = create_user(:password_confirmation => nil)
      assert u.errors.on(:password_confirmation)
    end
  end

  def test_should_require_email
    assert_no_difference 'User.count' do
      u = create_user(:email => nil)
      assert u.errors.on(:email)
    end
  end

  def test_should_reset_password
    users(:quentin).update_attributes(:password => 'new password', :password_confirmation => 'new password')
		assert_equal users(:quentin), User.authenticate('quentin@example.com', 'new password')
  end

  def test_should_not_rehash_password
    users(:quentin).update_attributes(:email => 'quentin2@example.com')
    assert_equal users(:quentin), User.authenticate('quentin2@example.com', 'test')
  end

  def test_should_authenticate_user
    assert_equal users(:quentin), User.authenticate('quentin@example.com', 'test')
  end

  def test_should_set_remember_token
    users(:quentin).remember_me
    assert_not_nil users(:quentin).remember_token
    assert_not_nil users(:quentin).remember_token_expires_at
  end

  def test_should_unset_remember_token
    users(:quentin).remember_me
    assert_not_nil users(:quentin).remember_token
    users(:quentin).forget_me
    assert_nil users(:quentin).remember_token
  end

	def test_username_should_have_valid_length
		too_long = ""
		41.times do |i|
			too_long += "A"
		end
		
		# too short
		u = create_user(:username => "tt")
		assert u.errors.on(:username)
		
		# too long
		u = create_user(:username => too_long)
		assert u.errors.on(:username)
	end
	
	def test_password_should_have_valid_length
		too_long = ""
		41.times do |i|
			too_long += "A"
		end
		
		# too short
		u = create_user(:password => "tt")
		assert u.errors.on(:password)
		
		# too long
		u = create_user(:password => too_long)
		assert u.errors.on(:password)
	end

	protected
	def create_user(options = {})
		User.create({
			:username => 'quire',
			:name => 'quire',
			:surname => 'qi',
			:birth_date => Time.now,
			:email => 'quire@example.com',
			:password => 'quire',
			:password_confirmation => 'quire'
			}.merge(options))
	end
end
